@@ -10,8 +10,12 @@
 #include <sys/select.h>
 #include <time.h>
 #include <unistd.h>
-#include <X11/Xlib.h>
-#include <wayland-client.h>
+#ifdef WITH_X11
+# include <X11/Xlib.h>
+#endif
+#ifdef WITH_WAYLAND
+# include <wayland-client.h>
+#endif
 
 /* Defines */
 #define MAXSLOTS 20
@@ -80,11 +84,12 @@ Distance pendingdistance;
 double xstart[MAXSLOTS], xend[MAXSLOTS], ystart[MAXSLOTS], yend[MAXSLOTS];
 unsigned nfdown = 0, nfpendingswipe = 0;
 struct timespec timedown;
-static Display *dpy;
 static int screen;
+#ifdef WITH_WAYLAND
 struct wl_display *wl_display;
 struct wl_registry *wl_registry;
 struct wl_output *wl_output;
+#endif
 static int screenwidth = 0, screenheight = 0;
 
 void
@@ -416,7 +421,8 @@ run()
 		.close_restricted = libinputcloserestricted,
 	};
 
-	li = libinput_path_create_context(&interface, NULL);
+	if ((li = libinput_path_create_context(&interface, NULL)) == NULL)
+		die("Failed to initialize context");
 
 	if ((d = libinput_path_add_device(li, device)) == NULL) {
 		die("Couldn't bind event from dev filesystem");
@@ -455,6 +461,7 @@ run()
 	libinput_unref(li);
 }
 
+#ifdef WITH_WAYLAND
 static void
 display_handle_geometry(void *data, struct wl_output *wl_output, int x, int y, int physical_width, int physical_height, int subpixel, const char *make, const char *model, int transform)
 {
@@ -514,6 +521,7 @@ wl_registry_listener wl_registry_listener = {
 	.global = registry_global,
 	.global_remove = registry_global_remove,
 };
+#endif
 
 int
 main(int argc, char *argv[])
@@ -521,7 +529,7 @@ main(int argc, char *argv[])
 	int i, j;
 	char *gestpt;
 
-	gestsarr = malloc(0);
+	gestsarr = NULL;
 	gestsarrlen = 0;
 
 	prctl(PR_SET_PDEATHSIG, SIGTERM);
@@ -617,12 +625,18 @@ main(int argc, char *argv[])
 	// Get display size (if not set with -w/-h)
 	if (screenwidth == 0 && screenheight == 0) {
 		if (getenv("WAYLAND_DISPLAY")) {
+#ifdef WITH_WAYLAND
 			wl_display = wl_display_connect(NULL);
 			wl_registry = wl_display_get_registry(wl_display);
 			wl_registry_add_listener(wl_registry, &wl_registry_listener, NULL);
 			wl_display_roundtrip(wl_display);
 			wl_display_dispatch(wl_display);
+#else
+			die("Wayland environment detected but support for it is not enabled");
+#endif
 		} else if (getenv("DISPLAY")) {
+#ifdef WITH_X11
+			Display *dpy;
 			if (!(dpy = XOpenDisplay(0))) {
 				die("Cannot open X display");
 			}
@@ -634,6 +648,9 @@ main(int argc, char *argv[])
 				screenwidth = DisplayHeight(dpy, screen);
 				screenheight = DisplayWidth(dpy, screen);
 			}
+#else
+			die("X11 environment detected but support for it is not enabled");
+#endif
 		} else {
 			die("Cannot detect display environment ($DISPLAY and $WAYLAND_DISPLAY unset); and no -w / -h screen geometry parameter options set");
 		}
@@ -642,6 +659,10 @@ main(int argc, char *argv[])
 	// E.g. no gestures passed on CLI - used gestures defined in config.def.h
 	if (gestsarrlen == 0) {
 		gestsarr = malloc(sizeof(gestures));
+		if (gestsarr == NULL) {
+			perror("Could not allocate memory");
+			exit(EXIT_FAILURE);
+		}
 		gestsarrlen = sizeof(gestures) / sizeof(Gesture);
 		memcpy(gestsarr, gestures, sizeof(gestures));
 	}
